package com.wechi.adweb.bridge.google.analytics.controller;

import com.google.gson.Gson;
import com.wechi.adweb.bridge.common.APIStatus;
import com.wechi.adweb.bridge.common.BaseController;
import com.wechi.adweb.bridge.common.OpenAPIRequest;
import com.wechi.adweb.bridge.common.OpenAPIResponse;
import com.wechi.adweb.bridge.exception.DataException;
import com.wechi.adweb.bridge.google.analytics.dto.report.GAReportRequestDTO;
import com.wechi.adweb.bridge.google.analytics.dto.report.ReportType;
import com.wechi.adweb.bridge.google.analytics.dto.report.data.GAReportDataDTO;
import com.wechi.adweb.bridge.google.analytics.service.GADataService;

import lombok.extern.slf4j.Slf4j;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;
import java.util.Objects;

/**
 * @author wfansh
 */
@Slf4j
@RestController
@RequestMapping("/api/google/ga")
public class GAController extends BaseController {
    @Autowired private GADataService gaDataService;

    private final Gson gson = new Gson();

    @RequestMapping("/report")
    public OpenAPIResponse<List<? extends GAReportDataDTO>> getGAReport(
            @RequestBody OpenAPIRequest<GAReportRequestDTO> apiRequest) throws DataException {
        long start = System.currentTimeMillis();
        log.info("****** getGAReport() ****** apiRequest = {}", gson.toJson(apiRequest));
        GAReportRequestDTO reportRequest = apiRequest.getData();

        // 1. Validates request parameters.
        if (Objects.isNull(reportRequest)
                || StringUtils.isEmpty(reportRequest.getPropertyName())
                || Objects.isNull(reportRequest.getReportType())
                || StringUtils.isEmpty(reportRequest.getStartDate())
                || StringUtils.isEmpty(reportRequest.getEndDate())
                || (ReportType.ADWEB_CUSTOM_REPORT.equals(reportRequest.getReportType())
                        && CollectionUtils.isEmpty(reportRequest.getMetrics())
                        && CollectionUtils.isEmpty(reportRequest.getDimensions()))) {
            return OpenAPIResponse.<List<? extends GAReportDataDTO>>builder()
                    .status(APIStatus.BAD_REQUEST)
                    .message("Invalid request parameters")
                    .build();
        }

        // 2. Executes API requests.
        List<? extends GAReportDataDTO> report = gaDataService.runReport(apiRequest.getData());
        log.info("****** getGAReport() ****** duration = {} seconds", getElapsedSeconds(start));
        return OpenAPIResponse.<List<? extends GAReportDataDTO>>builder()
                .status(APIStatus.SUCCESS)
                .data(report)
                .build();
    }
}
