package com.wechi.adweb.bridge.google.analytics;

import static com.wechi.adweb.bridge.google.analytics.dto.report.ReportConstant.*;

import com.google.gson.Gson;
import com.wechi.adweb.bridge.exception.DataException;
import com.wechi.adweb.bridge.google.analytics.dto.report.GAReportRequestDTO;
import com.wechi.adweb.bridge.google.analytics.dto.report.ReportType;
import com.wechi.adweb.bridge.google.analytics.service.GAAdminService;
import com.wechi.adweb.bridge.google.analytics.service.GADataService;

import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;

import java.util.List;

@SpringBootTest
public class GAServiceTests {

    @Autowired private GAAdminService gaAdminService;
    @Autowired private GADataService gaDataService;

    private final String propertyName = "properties/457183952";
    private final String startDate = "2023-10-01";
    private final String endDate = "2024-10-01";
    private final Gson gson = new Gson();

    @Test
    void runReport() throws DataException {

        // 1. Country chart.
        GAReportRequestDTO reportRequest = new GAReportRequestDTO();
        reportRequest.setPropertyName(propertyName);
        reportRequest.setReportType(ReportType.ADWEB_COUNTRY_CHART);
        reportRequest.setStartDate(startDate);
        reportRequest.setEndDate(endDate);
        System.out.println(gson.toJson(gaDataService.runReport(reportRequest)));

        // 2. Session source medium view.
        reportRequest = new GAReportRequestDTO();
        reportRequest.setPropertyName(propertyName);
        reportRequest.setReportType(ReportType.ADWEB_SESSION_SOURCE_MEDIUM_VIEW);
        reportRequest.setStartDate(startDate);
        reportRequest.setEndDate(endDate);
        System.out.println(gson.toJson(gaDataService.runReport(reportRequest)));

        // 3. Page path view.
        reportRequest = new GAReportRequestDTO();
        reportRequest.setPropertyName(propertyName);
        reportRequest.setReportType(ReportType.ADWEB_PAGE_PATH_VIEW);
        reportRequest.setStartDate(startDate);
        reportRequest.setEndDate(endDate);
        System.out.println(gson.toJson(gaDataService.runReport(reportRequest)));

        // 4. Date view.
        reportRequest = new GAReportRequestDTO();
        reportRequest.setPropertyName(propertyName);
        reportRequest.setReportType(ReportType.ADWEB_DATE_VIEW);
        reportRequest.setStartDate(startDate);
        reportRequest.setEndDate(endDate);
        System.out.println(gson.toJson(gaDataService.runReport(reportRequest)));

        // 5. Custom report.
        reportRequest = new GAReportRequestDTO();
        reportRequest.setPropertyName(propertyName);
        reportRequest.setReportType(ReportType.ADWEB_CUSTOM_REPORT);
        reportRequest.setStartDate(startDate);
        reportRequest.setEndDate(endDate);
        reportRequest.setMetrics(
                List.of(
                        METRIC_TOTAL_USERS,
                        METRIC_ACTIVE_USERS,
                        METRIC_SCREEN_PAGE_VIEWS,
                        METRIC_BOUNCE_RATE,
                        METRIC_AVG_SESSION_DURATION));
        reportRequest.setOrderBy(METRIC_TOTAL_USERS);
        reportRequest.setOrderByType(GAReportRequestDTO.OrderByType.METRICS);
        reportRequest.setDimensions(List.of(DIMENSION_DATE));
        reportRequest.setOrderByDesc(true);
        System.out.println(gson.toJson(gaDataService.runReport(reportRequest)));
    }
}
